/***************************************************************************
 * Copyright 2012 by
 *  + Christian-Albrechts-University of Kiel
 *    + Department of Computer Science
 *      + Software Engineering Group 
 *  and others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.view;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;

import kieker.analysis.AnalysisController;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.beans.application.ProjectsBean;
import kieker.webgui.common.IProjectManagerFacade;
import kieker.webgui.common.ProjectManagerFacade;
import kieker.webgui.common.exception.AnalysisInitializationException;
import kieker.webgui.common.exception.AnalysisStateException;
import kieker.webgui.common.exception.ProjectNotExistingException;

/**
 * /**
 * The {@link CurrentControllerBean} contains the necessary data behind an instance of the analysis controller. The class is a JSF managed bean with view scope to
 * make sure that one user (even in one session) can open multiple projects at a time without causing any problems.
 * 
 * @author Nils Christian Ehmke
 * @version 1.0
 */
@ManagedBean
@ViewScoped
public class CurrentControllerBean {

	private static final Log LOG = LogFactory.getLog(CurrentControllerBean.class);
	private final IProjectManagerFacade projectManagerFacade = ProjectManagerFacade.getInstance();
	private final ArrayList<String> logEntries = new ArrayList<String>();
	private String projectName;
	@ManagedProperty(value = "#{projectsBean}")
	private ProjectsBean projectsBean;

	/**
	 * Creates a new instance of this class. <b>Do not use this constructor. This bean is JSF managed.</b>
	 */
	public CurrentControllerBean() {
		// No code necessary
	}

	/**
	 * The getter for the {@link #projectsBean}-property.
	 * 
	 * @return {@link #projectsBean}
	 */
	public ProjectsBean getProjectsBean() {
		return this.projectsBean;
	}

	/**
	 * The setter for the {@link #projectsBean}-property.
	 * 
	 * @param projectsBean
	 *            The new value for {@link #projectsBean}.
	 */
	public void setProjectsBean(final ProjectsBean projectsBean) {
		this.projectsBean = projectsBean;
	}

	/**
	 * This method sets the project stored within this bean and returns the new page for the navigation.
	 * 
	 * @param newName
	 *            The name of the project.
	 */
	public void setProjectName(final String newName) {
		synchronized (this) {
			// Remember the given parameters
			this.projectName = newName;
		}
	}

	/**
	 * This method delivers the project name stored in this bean.
	 * 
	 * @return The project name for this user.
	 */
	public String getProjectName() {
		synchronized (this) {
			return this.projectName;
		}
	}

	/**
	 * This method starts the current analysis and informs the user about a fail.
	 */
	public void startAnalysis() {
		synchronized (this) {
			this.addLogEntry("Starting Analysis for project '" + this.projectName + "'");
			try {
				this.projectManagerFacade.startAnalysis(this.projectName);
			} catch (final AnalysisStateException ex) {
				CurrentControllerBean.LOG.info("The analysis has already been started.", ex);
				this.addLogEntry(ex);
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				this.addLogEntry(ex);
			}
		}
	}

	/**
	 * This method stops the current analysis and informs the user about a fail.
	 */
	public void stopAnalysis() {
		try {
			this.addLogEntry("Stopping Analysis for project '" + this.projectName + "'");
			synchronized (this) {
				this.projectManagerFacade.stopAnalysis(this.projectName);
			}
		} catch (final AnalysisStateException ex) {
			CurrentControllerBean.LOG.info("The analysis has not been started yet.", ex);
			this.addLogEntry(ex);
		} catch (final ProjectNotExistingException ex) {
			CurrentControllerBean.LOG.info("The project does not exist.", ex);
			this.addLogEntry(ex);
		}
	}

	/**
	 * This method initializes the current analysis and informs the user about a fail.
	 */
	public void instantiateAnalysis() {
		synchronized (this) {
			this.addLogEntry("Instantiating Analysis for project '" + this.projectName + "'");
			try {
				this.projectManagerFacade.initializeAnalysis(this.projectName, this.projectManagerFacade.getClassLoader(this.projectName)); // NOPMD (ClassLoader)
			} catch (final AnalysisStateException ex) {
				CurrentControllerBean.LOG.error("The analysis has already been instantiated.", ex);
				this.addLogEntry(ex);
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				this.addLogEntry(ex);
			} catch (final IOException ex) {
				CurrentControllerBean.LOG.info("An error occured during the initialization.", ex);
				this.addLogEntry(ex);
			} catch (final AnalysisInitializationException ex) {
				CurrentControllerBean.LOG.info("An error occured during the initialization.", ex);
				this.addLogEntry(ex);
			}
		}
	}

	/**
	 * This method cleans the current analysis instance.
	 */
	public void cleanAnalysis() {
		synchronized (this) {
			this.addLogEntry("Cleaning Analysis for project '" + this.projectName + "'");
			try {
				this.projectManagerFacade.cleanAnalysis(this.projectName);
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				this.addLogEntry(ex);
			} catch (final AnalysisStateException ex) {
				CurrentControllerBean.LOG.error("The analysis has not been instantiated yet.", ex);
				this.addLogEntry(ex);
			}
		}
	}

	/**
	 * Checks whether the analysis is currently running.
	 * 
	 * @return true if and only if the analysis is running.
	 */
	public boolean isAnalysisRunning() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.RUNNING;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * Checks whether the analysis is currently in the ready state.
	 * 
	 * @return true if and only if the analysis is ready to be started.
	 */
	public boolean isAnalysisReady() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.READY;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * Checks whether the analysis is not available.
	 * 
	 * @return true if and only if the analysis is <b>not</b> available.
	 */
	public boolean isAnalysisNotAvailable() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == null;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return true;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return true;
			}
		}
	}

	/**
	 * Checks whether the analysis is currently terminated.
	 * 
	 * @return true if and only if the analysis has been terminated.
	 */
	public boolean isAnalysisTerminated() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.TERMINATED;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	/**
	 * Checks whether the analysis is currently in the failed state.
	 * 
	 * @return true if and only if the analysis has failed.
	 */
	public boolean isAnalysisFailed() {
		synchronized (this) {
			try {
				return this.projectManagerFacade.getCurrentState(this.projectName) == AnalysisController.STATE.FAILED;
			} catch (final ProjectNotExistingException ex) {
				CurrentControllerBean.LOG.info("The project does not exist.", ex);
				return false;
			} catch (final NullPointerException ex) {
				// This exception can occur, when the projectsBean has not been initialized
				CurrentControllerBean.LOG.warn("A null pointer exception occured.", ex);
				return false;
			}
		}
	}

	public Collection<String> getLog() {
		return this.logEntries;
	}

	private void addLogEntry(final Throwable cause) {
		final Writer result = new StringWriter();
		final PrintWriter printWriter = new PrintWriter(result);
		cause.printStackTrace(printWriter);
		this.addLogEntry(result.toString());
	}

	private void addLogEntry(final String msg) {
		final String finalMsg = new Date().toString() + " : " + msg;
		synchronized (this) {
			if (this.logEntries.size() > 50) {
				this.logEntries.remove(0);
			}
			this.logEntries.add(finalMsg);
		}
	}
}
